package org.klomp.eclipse.team.sftp.ui;

import java.util.Properties;

import org.eclipse.core.runtime.IProgressMonitor;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.team.internal.ui.target.TargetWizardPage;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.resource.ImageDescriptor;
import org.klomp.eclipse.team.sftp.SftpPlugin;
import org.klomp.eclipse.team.sftp.target.SftpSite;

/**
 * Wizard page for configuring a project with an Sftp location.
 */
public class SftpMainPage extends TargetWizardPage
{
    private final Listener validationListener = new Listener()
    {
        public void handleEvent(Event e)
        {
            SftpMainPage.this.validateFields();
        }
    };

    private Text usernameWidget;
    private Combo hostnameWidget;

    private Combo portWidget;
    private Label serverCmdLabel;
    private Combo serverCmdWidget;
    private Combo basePathWidget;
    private Text passwordWidget;
    private Text timeoutWidget;

    private Combo sshVersion;

    // Dialog store id constants
    private static final String STORE_HOSTNAME_ID = "SftpWizardMainPage.STORE_HOSTNAME_ID";
    private static final String STORE_PORT_ID = "SftpWizardMainPage.STORE_PORT_ID";
    private static final String STORE_SERVERCMD_ID = "SftpWizardMainPage.STORE_SERVERCMD_ID";
    private static final String STORE_BASEPATH_ID = "SftpWizardMainPage.STORE_BASEPATH_ID";

    /**
     * In case the page was launched from a different wizard.
     */
    private IDialogSettings settings;

    public SftpMainPage(
        String pageName,
        String title,
        String description,
        ImageDescriptor titleImage)
    {
        super(pageName, title, titleImage);
        setDescription(description);
        setTitle(title);
    }

    protected void setDialogSettings(IDialogSettings settings)
    {
        this.settings = settings;
    }

    protected IDialogSettings getDialogSettings()
    {
        return settings;
    }

    /*
     * Creates the UI part of the page.
     * 
     * @param parent  the parent of the created widgets
     */
    public void createControl(Composite parent)
    {
        Composite composite = createComposite(parent, 4);

        Label label;

        label = createLabel(composite, SftpPlugin.getString("%SftpWizardMainPage.host")); //$NON-NLS-1$
        setHorizontalSpan(label, 1);

        hostnameWidget = createEditableCombo(composite);
        setHorizontalSpan(hostnameWidget, 3);
        hostnameWidget.addListener(SWT.Modify, validationListener);

        createLabel(composite, SftpPlugin.getString("%SftpWizardMainPage.port")); //$NON-NLS-1$
        portWidget = createEditableCombo(composite);
        portWidget.setTextLimit(4);
        setWidthHint(portWidget, 50);
        portWidget.addListener(SWT.Modify, validationListener);

        createLabel(composite, SftpPlugin.getString("%SftpWizardMainPage.sshVersion"));
        // $NON-NLS-1$
        sshVersion = new Combo(composite, SWT.READ_ONLY | SWT.DROP_DOWN);
        sshVersion.add("SSH1");
        sshVersion.add("SSH2");

        sshVersion.select(1);
        sshVersion.addSelectionListener(new SelectionListener()
        {
            public void widgetDefaultSelected(SelectionEvent e)
            {
                widgetSelected(e);
            }

            public void widgetSelected(SelectionEvent e)
            {

                SftpMainPage.this.validateFields();
                if (sshVersion.getSelectionIndex() == 0)
                {
                    serverCmdLabel.setEnabled(true);
                    serverCmdWidget.setEnabled(true);
                }
                else
                {
                    serverCmdLabel.setEnabled(false);
                    serverCmdWidget.setEnabled(false);
                }
            }
        });

        serverCmdLabel = createLabel(composite, SftpPlugin.getString("%SftpWizardMainPage.serverCmd")); //$NON-NLS-1$
        setHorizontalSpan(serverCmdLabel, 1);
        serverCmdLabel.setEnabled(false);
        serverCmdWidget = createEditableCombo(composite);
        setHorizontalSpan(serverCmdWidget, 3);
        serverCmdWidget.setEnabled(false);
        serverCmdWidget.addListener(SWT.Modify, validationListener);

        label = createLabel(composite, SftpPlugin.getString("%SftpWizardMainPage.basePath")); //$NON-NLS-1$
        setHorizontalSpan(label, 1);
        basePathWidget = createEditableCombo(composite);
        setHorizontalSpan(basePathWidget, 3);
        basePathWidget.addListener(SWT.Modify, validationListener);

        label = createLabel(composite, SftpPlugin.getString("%SftpWizardMainPage.userName")); //$NON-NLS-1$
        setHorizontalSpan(label, 1);
        usernameWidget = createTextField(composite);
        setWidthHint(usernameWidget, 50);
        setHorizontalSpan(usernameWidget, 1);
        usernameWidget.addListener(SWT.Modify, validationListener);

        label = createLabel(composite, SftpPlugin.getString("%SftpWizardMainPage.password")); //$NON-NLS-1$
        setHorizontalSpan(label, 1);
        passwordWidget = createTextField(composite);
        setWidthHint(passwordWidget, 50);
        setHorizontalSpan(passwordWidget, 1);
        passwordWidget.setEchoChar('*');

        createLabel(composite, SftpPlugin.getString("%SftpWizardMainPage.timeout")); //$NON-NLS-1$
        timeoutWidget = createTextField(composite);
        timeoutWidget.setTextLimit(3);
        setWidthHint(timeoutWidget, 50);
        timeoutWidget.addListener(SWT.Modify, validationListener);
        createLabel(composite, SftpPlugin.getString("%SftpWizardMainPage.seconds")); //$NON-NLS-1$

        //        Composite passiveButtonTabGroup = new Composite(composite, SWT.NONE);
        //        GridData data = new GridData();
        //        data.horizontalSpan = 2;
        //        passiveButtonTabGroup.setLayoutData(data);
        //        passiveButtonTabGroup.setLayout(new FillLayout());

        initializeValues();
        validateFields();
        hostnameWidget.setFocus();
        setControl(composite);
    }

    private void setHorizontalSpan(Control c, int span)
    {
        GridData d = (GridData) c.getLayoutData();
        d.horizontalSpan = span;
    }

    private void setWidthHint(Control c, int width)
    {
        GridData d = (GridData) c.getLayoutData();
        d.widthHint = width;
    }

    public SftpSite getSite()
    {
        if (!isPageComplete())
        {
            return null;
        }

        return new SftpSite(getCurrentValues());
    }

    private Properties getCurrentValues()
    {
        Properties result = new Properties();
        result.setProperty("host", hostnameWidget.getText()); //$NON-NLS-1$
        result.setProperty("port", portWidget.getText()); //$NON-NLS-1$
        result.setProperty("serverCmd", serverCmdWidget.getText()); //$NON-NLS-1$
        result.setProperty("sshVersion", String.valueOf(sshVersion.getSelectionIndex() + 1));
        result.setProperty("basePath", basePathWidget.getText()); //$NON-NLS-1$
        result.setProperty("username", usernameWidget.getText()); //$NON-NLS-1$
        result.setProperty("password", passwordWidget.getText()); //$NON-NLS-1$
        result.setProperty("connectionTimeout", timeoutWidget.getText()); //$NON-NLS-1$
        return result;
    }

    /*
     * @see WizardPage#finish
     */
    public boolean finish(IProgressMonitor monitor)
    {
        // Set the result to be the current values
        saveWidgetValues();
        return true;
    }

    /*
     * Initializes states of the controls.
     */
    private void initializeValues()
    {
        IDialogSettings settings = getDialogSettings();
        //        System.err.println("Settings#2: " + settings);
        if (settings != null)
        {
            String[] hostNames = settings.getArray(STORE_HOSTNAME_ID);
            if (hostNames != null)
            {
                for (int i = 0; i < hostNames.length; i++)
                {
                    hostnameWidget.add(hostNames[i]);
                }
            }
            String[] ports = settings.getArray(STORE_PORT_ID);
            if (ports != null)
            {
                for (int i = 0; i < ports.length; i++)
                {
                    portWidget.add(ports[i]);
                }
            }
            portWidget.setText(String.valueOf(SftpSite.DEFAULT_PORT));

            String[] serverCmds = settings.getArray(STORE_SERVERCMD_ID);
            if (serverCmds != null)
            {
                for (int i = 0; i < serverCmds.length; i++)
                {
                    serverCmdWidget.add(serverCmds[i]);
                }
            }
            serverCmdWidget.setText(SftpPlugin.DEFAULT_SERVER_CMD);

            String[] basePaths = settings.getArray(STORE_BASEPATH_ID);
            if (basePaths != null)
            {
                for (int i = 0; i < basePaths.length; i++)
                {
                    basePathWidget.add(basePaths[i]);
                }
            }

            timeoutWidget.setText(String.valueOf(SftpSite.DEFAULT_TIMEOUT));
        }
    }

    /*
     * Saves the widget values
     */
    private void saveWidgetValues()
    {
        // Update history
        IDialogSettings settings = getDialogSettings();
        if (settings != null)
        {
            String[] hostNames = settings.getArray(STORE_HOSTNAME_ID);
            if (hostNames == null)
                hostNames = new String[0];
            hostNames = addToHistory(hostNames, hostnameWidget.getText());
            settings.put(STORE_HOSTNAME_ID, hostNames);

            String[] ports = settings.getArray(STORE_PORT_ID);
            if (ports == null)
                ports = new String[0];
            ports = addToHistory(ports, portWidget.getText());
            settings.put(STORE_PORT_ID, ports);

            String[] serverCmds = settings.getArray(STORE_SERVERCMD_ID);
            if (serverCmds == null)
                serverCmds = new String[0];
            serverCmds = addToHistory(serverCmds, basePathWidget.getText());
            settings.put(STORE_SERVERCMD_ID, serverCmds);

            String[] basePaths = settings.getArray(STORE_BASEPATH_ID);
            if (basePaths == null)
                basePaths = new String[0];
            basePaths = addToHistory(basePaths, basePathWidget.getText());
            settings.put(STORE_BASEPATH_ID, basePaths);
        }
    }

    /**
     * Validates the contents of the editable fields and set page completion 
     * and error messages appropriately.
     */
    private void validateFields()
    {
        setMessage("");

        String host = hostnameWidget.getText();
        if (host.length() == 0)
        {
            setMessage(SftpPlugin.getString("%SftpWizardMainPage.noHost"), WARNING); //$NON-NLS-1$
            setPageComplete(false);
            return;
        }

        // validate the port
        String port = portWidget.getText();
        try
        {
            int portInt = Integer.parseInt(port);
            if (portInt <= 0)
            {
                setMessage(SftpPlugin.getString("%SftpWizardMainPage.invalidPort"), WARNING); //$NON-NLS-1$
                setPageComplete(false);
                return;
            }
        }
        catch (NumberFormatException e)
        {
            setMessage(SftpPlugin.getString("%SftpWizardMainPage.invalidPort"), WARNING); //$NON-NLS-1$
            setPageComplete(false);
            return;
        }

        String serverCmd = serverCmdWidget.getText();
        if (serverCmd.length() == 0)
        {
            setMessage(SftpPlugin.getString("%SftpWizardMainPage.noServerCmd"), WARNING); //$NON-NLS-1$
            setPageComplete(false);
            return;
        }

        String username = usernameWidget.getText();
        if (username.length() == 0)
        {
            setPageComplete(false);
            setMessage(SftpPlugin.getString("%SftpWizardMainPage.usernameMustBeSpecified"), WARNING); //$NON-NLS-1$
            return;
        }

        // validate the timeout
        String timeout = timeoutWidget.getText();
        try
        {
            int timeoutInt = Integer.parseInt(timeout);
            if (timeoutInt <= 0)
            {
                setMessage(SftpPlugin.getString("%SftpWizardMainPage.invalidTimeout"), WARNING); //$NON-NLS-1$
                setPageComplete(false);
                return;
            }
        }
        catch (NumberFormatException e)
        {
            setMessage(SftpPlugin.getString("%SftpWizardMainPage.invalidTimeout"), WARNING); //$NON-NLS-1$
            setPageComplete(false);
            return;
        }

        setMessage(null);
        setPageComplete(true);
    }

    public void setVisible(boolean visible)
    {
        super.setVisible(visible);
        if (visible)
        {
            hostnameWidget.setFocus();
        }
    }
}