package org.klomp.sftp.packet;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.klomp.ssh.DataIO;

public class Attrs
{
    public final static int FILEXFER_ATTR_SIZE = 0x00000001;
    public final static int FILEXFER_ATTR_UIDGID = 0x00000002;
    public final static int FILEXFER_ATTR_PERMISSIONS = 0x00000004;
    public final static int FILEXFER_ATTR_ACCESSTIME = 0x00000008;
    public final static int FILEXFER_ATTR_CREATETIME = 0x00000010;
    public final static int FILEXFER_ATTR_MODIFYTIME = 0x00000020;
    public final static int FILEXFER_ATTR_ACL = 0x00000040;
    public final static int FILEXFER_ATTR_OWNERGROUP = 0x00000080;
    public final static int FILEXFER_ATTR_SUBSECOND_TIMES = 0x00000100;
    public final static int FILEXFER_ATTR_EXTENDED = 0x80000000;

    public final static byte FILEXFER_TYPE_REGULAR = 1;
    public final static byte FILEXFER_TYPE_DIRECTORY = 2;
    public final static byte FILEXFER_TYPE_SYMLINK = 3;
    public final static byte FILEXFER_TYPE_SPECIAL = 4;
    public final static byte FILEXFER_TYPE_UNKNOWN = 5;

    private int flags;
    private byte type;
    private long size;
    private int uid;
    private int gid;
    private String owner;
    private String group;
    private int permissions;
    private long accessTime;
    private int accessTimeNanos;
    private long createTime;
    private int createTimeNanos;
    private long modifyTime;
    private int modifyTimeNanos;
    private List acl;
    private int extended_count;

    private List extendedTypes;
    private Map extensions;

    protected void readAttrs(InputStream payload, int version) throws IOException
    {
        flags = DataIO.readInt(payload);
        
        // V2 has no type field
        if (version > 3)
        {
            type = (byte) payload.read();
        }

        if ((flags & FILEXFER_ATTR_SIZE) != 0)
        {
            size = DataIO.readLong(payload);
        }
        
        if (version <= 3 && (flags & FILEXFER_ATTR_UIDGID) != 0)
        {
            uid = DataIO.readInt(payload);
            gid = DataIO.readInt(payload);
        }
        
        if ((flags & FILEXFER_ATTR_OWNERGROUP) != 0)
        {
            owner = DataIO.readStringAscii(payload);
            group = DataIO.readStringAscii(payload);
        }
        
        if ((flags & FILEXFER_ATTR_PERMISSIONS) != 0)
        {
            permissions = DataIO.readInt(payload);
        }
        
        if ((flags & FILEXFER_ATTR_ACCESSTIME) != 0)
        {
            if (version > 3)
            {
                accessTime = DataIO.readLong(payload); // V4
            }
            else
            {
                // In versions up to 3, this flag was called FILEXFER_ATTR_ACMODTIME,
                // and included both access and modification times.
                accessTime = DataIO.readInt(payload); // V3
                modifyTime = DataIO.readInt(payload); // V2??
            }

            if ((flags & FILEXFER_ATTR_SUBSECOND_TIMES) != 0)
            {
                accessTimeNanos = DataIO.readInt(payload);
            }
        }
        
        if ((flags & FILEXFER_ATTR_CREATETIME) != 0)
        {
            if (version > 3)
            {
                createTime = DataIO.readLong(payload); // V4
            }
            else
            {
                createTime = DataIO.readInt(payload); // V3
            }

            if ((flags & FILEXFER_ATTR_SUBSECOND_TIMES) != 0)
            {
                createTimeNanos = DataIO.readInt(payload);
            }
        }
        
        if ((flags & FILEXFER_ATTR_MODIFYTIME) != 0)
        {
            if (version > 3)
            {
                modifyTime = DataIO.readLong(payload); // V4
            }
            else
            {
                modifyTime = DataIO.readInt(payload); // V3
            }

            if ((flags & FILEXFER_ATTR_SUBSECOND_TIMES) != 0)
            {
                modifyTimeNanos = DataIO.readInt(payload);
            }
        }
        
        if ((flags & FILEXFER_ATTR_ACL) != 0)
        {
            int aceCount = DataIO.readInt(payload);
            acl = new ArrayList(aceCount);
            for (int i = 0; i < aceCount; i++)
            {
                AcEntry ace = new AcEntry();
                ace.readAcEntry(payload, version);
                acl.add(ace);
            }
        }

        if ((flags & FILEXFER_ATTR_EXTENDED) != 0)
        {
            int extendedCount = DataIO.readInt(payload);

            extendedTypes = new ArrayList(extendedCount);
            extensions = new HashMap();
            for (int i = 0; i < extendedCount; i++)
            {
                String extendedType = DataIO.readStringUTF8(payload);
                String extensionData = DataIO.readStringUTF8(payload);
                extendedTypes.add(extendedType);
                extensions.put(extendedType, extensionData);
            }
        }
    }

    protected void writeAttrs(ByteArrayOutputStream payload, int version)
    {
        DataIO.writeInt(payload, flags);

        if (version > 3)
        {
            payload.write(type);
        }

        if ((flags & FILEXFER_ATTR_SIZE) != 0)
        {
            DataIO.writeLong(payload, size);
        }

        if (version <= 3 && (flags & FILEXFER_ATTR_OWNERGROUP) != 0)
        {
            DataIO.writeStringAscii(payload, owner);
            DataIO.writeStringAscii(payload, group);

        }

        if ((flags & FILEXFER_ATTR_PERMISSIONS) != 0)
        {
            DataIO.writeInt(payload, permissions);
        }

        if ((flags & FILEXFER_ATTR_ACCESSTIME) != 0)
        {
            if (version > 3)
            {
                DataIO.writeLong(payload, accessTime); // V4
            }
            else
            {
                DataIO.writeInt(payload, (int) accessTime); // V3
                DataIO.writeInt(payload, (int) modifyTime); // V3
            }

            if ((flags & FILEXFER_ATTR_SUBSECOND_TIMES) != 0)
            {
                DataIO.writeInt(payload, accessTimeNanos);
            }
        }

        if ((flags & FILEXFER_ATTR_CREATETIME) != 0)
        {
            if (version > 3)
            {
                DataIO.writeLong(payload, createTime); // V4
            }
            else
            {
                DataIO.writeInt(payload, (int) createTime); // V3
            }

            if ((flags & FILEXFER_ATTR_SUBSECOND_TIMES) != 0)
            {
                DataIO.writeInt(payload, createTimeNanos);
            }
        }

        if ((flags & FILEXFER_ATTR_MODIFYTIME) != 0)
        {
            if (version > 3)
            {
                DataIO.writeLong(payload, modifyTime); // V4
            }
            else
            {
                DataIO.writeInt(payload, (int) modifyTime); // V3
            }

            if ((flags & FILEXFER_ATTR_SUBSECOND_TIMES) != 0)
            {
                DataIO.writeInt(payload, modifyTimeNanos);
            }
        }

        if ((flags & FILEXFER_ATTR_ACL) != 0)
        {
            DataIO.writeInt(payload, acl.size());
            for (Iterator i = acl.iterator(); i.hasNext();)
            {
                AcEntry entry = (AcEntry) i.next();
                entry.writeAcEntry(payload, version);
            }
        }

        if ((flags & FILEXFER_ATTR_EXTENDED) != 0)
        {
            DataIO.writeInt(payload, extendedTypes.size());
            for (Iterator i = extendedTypes.iterator(); i.hasNext();)
            {
                String extendedType = (String) i.next();
                DataIO.writeStringUTF8(payload, extendedType);
                DataIO.writeStringUTF8(payload, (String) extensions.get(extendedType));
            }
        }
    }

    public long getAccessTime()
    {
        return accessTime;
    }

    public int getAccessTimeNanos()
    {
        return accessTimeNanos;
    }

    public long getCreateTime()
    {
        return createTime;
    }

    public int getCreateTimeNanos()
    {
        return createTimeNanos;
    }

    public int getFlags()
    {
        return flags;
    }

    public String getGroup()
    {
        return group;
    }

    public long getModifyTime()
    {
        return modifyTime;
    }

    public int getModifyTimeNanos()
    {
        return modifyTimeNanos;
    }

    public String getOwner()
    {
        return owner;
    }

    public int getPermissions()
    {
        return permissions;
    }

    public long getSize()
    {
        return size;
    }

    public byte getType()
    {
        return type;
    }

    public void setAccessTime(long atime)
    {
        this.accessTime = atime;
    }

    public void setAccessTimeNanos(int atime_nseconds)
    {
        this.accessTimeNanos = atime_nseconds;
    }

    public void setCreateTime(long createtime)
    {
        this.createTime = createtime;
    }

    public void setCreateTimeNanos(int createtime_nseconds)
    {
        this.createTimeNanos = createtime_nseconds;
    }

    public void setFlags(int flags)
    {
        this.flags = flags;
    }

    public void setGroup(String group)
    {
        this.group = group;
    }

    public void setModifyTime(long mtime)
    {
        this.modifyTime = mtime;
    }

    public void setModifyTimeNanos(int mtime_nseconds)
    {
        this.modifyTimeNanos = mtime_nseconds;
    }

    public void setOwner(String owner)
    {
        this.owner = owner;
    }

    public void setPermissions(int permissions)
    {
        this.permissions = permissions;
    }

    public void setSize(long size)
    {
        this.size = size;
    }

    public void setType(byte type)
    {
        this.type = type;
    }

    public List getAcl()
    {
        return acl;
    }

    public void setAcl(List acl)
    {
        this.acl = acl;
    }

}