package org.klomp.sftp.packet;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import org.klomp.ssh.DataIO;

public class FxpOpen extends SftpPacket
{
    /**
     * Open the file for reading.
     * Used in the property <code>pflags</code> in an FXP_OPEN packet.
     */
    public final static int FXF_READ = 0x00000001;
    /**
     * Open the file for writing.  If both this and FXF_READ are
     * specified, the file is opened for both reading and writing.
     * Used in the property <code>pflags</code> in an FXP_OPEN packet.
     */
    public final static int FXF_WRITE = 0x00000002;
    /**
     * Force all writes to append data at the end of the file.
     * Used in the property <code>pflags</code> in an FXP_OPEN packet.
     */
    public final static int FXF_APPEND = 0x00000004;
    /**
     * If this flag is specified, then a new file will be created if one
     * does not alread exist (if FXF_TRUNC is specified, the new file will
     * be truncated to zero length if it previously exists).
     * Used in the property <code>pflags</code> in an FXP_OPEN packet.
     */
    public final static int FXF_CREAT = 0x00000008;
    /**
     * Forces an existing file with the same name to be truncated to zero
     * length when creating a file by specifying FXF_CREAT.
     * FXF_CREAT MUST also be specified if this flag is used.
     * Used in the property <code>pflags</code> in an FXP_OPEN packet.
     */
    public final static int FXF_TRUNC = 0x00000010;
    /**
     * Causes the request to fail if the named file already exists.
     * SSH_FXF_CREAT MUST also be specified if this flag is used.
     * Used in the property <code>pflags</code> in an FXP_OPEN packet.
     */
    public final static int FXF_EXCL = 0x00000020;
    /**
     * Indicates that the server should treat the file as text and
     * convert it to the canonical newline convention in use.
     * <p>
     * When a file is opened with the FXF_TEXT flag, the offset field in
     * both the read and write function are ignored.
     * <p>
     * Clients SHOULD use the FXF_APPEND flag to append data to a
     * text file rather then using write with a calculated offset.
     * <p>
     * Only available in protocol version 4 and above.
     * Used in the property <code>pflags</code> in an FXP_OPEN packet.
     */
    public final static int FXF_TEXT = 0x00000040;

    private String path;
    private int pflags;
    private Attrs attrs;

    FxpOpen(int version)
    {
        super(SftpPacket.FXP_OPEN, version);
    }

    FxpOpen(int version, int id)
    {
        super(SftpPacket.FXP_OPEN, version, id);
    }

    public String toString()
    {
        return super.toString() + " (path: " + path + ")";
    }

    protected void readPayload(InputStream payload) throws IOException
    {
        id = DataIO.readInt(payload);
        path = DataIO.readStringUTF8(payload);
        pflags = DataIO.readInt(payload);
        attrs = new Attrs();
        attrs.readAttrs(payload, version);
    }

    protected void writePayload(ByteArrayOutputStream payload)
    {
        DataIO.writeInt(payload, id);
        DataIO.writeStringUTF8(payload, path);
        DataIO.writeInt(payload, pflags);
        attrs.writeAttrs(payload, version);
    }
    public Attrs getAttrs()
    {
        return attrs;
    }

    public String getPath()
    {
        return path;
    }

    public int getPflags()
    {
        return pflags;
    }

    public void setAttrs(Attrs attrs)
    {
        this.attrs = attrs;
    }

    public void setPath(String filename)
    {
        this.path = filename;
    }

    public void setPflags(int pflags)
    {
        this.pflags = pflags;
    }
}