package org.klomp.sftp.protocol;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;

import org.klomp.eclipse.team.sftp.SftpException;
import org.klomp.eclipse.team.sftp.client.SftpCommunicationException;
import org.klomp.sftp.packet.Attrs;
import org.klomp.sftp.packet.FxpHandle;
import org.klomp.sftp.packet.FxpName;
import org.klomp.sftp.packet.FxpOpenDir;
import org.klomp.sftp.packet.FxpReadDir;
import org.klomp.sftp.packet.FxpStatus;
import org.klomp.sftp.packet.SftpPacket;
import org.klomp.sftp.packet.SftpPacketFactory;

public class DirectoryFile extends SftpFile {
    public final static int MAX_SEND_AT_ONCE_PACKETS = 20;
    private int[] sentPacketIds = new int[MAX_SEND_AT_ONCE_PACKETS];

    private DirectoryFile(SftpConnection connection, String path, byte[] handle) {
        super(connection, path, handle);
    }

    public static DirectoryFile openDir(SftpConnection connection, String directoryPath)
        throws SftpException {
        SftpPacketFactory pktFactory = connection.getPktFactory();
        FxpOpenDir pkt = pktFactory.createOpenDir();
        pkt.setPath(directoryPath);
        connection.sendPacket(pkt);
        SftpPacket response = connection.readPacket(pkt.getId());

        switch (response.getType()) {
            case SftpPacket.FXP_STATUS :
                defaultStatusHandler((FxpStatus) response);
                throw new SftpException("Unexpected non-error status received: " + response);
            case SftpPacket.FXP_HANDLE :
                FxpHandle handlePkt = (FxpHandle) response;
                byte[] handle = handlePkt.getHandle();
                return new DirectoryFile(connection, directoryPath, handle);
            default :
                throw new SftpException("Unexpected packet received: " + response);
        }
    }

    public Map list(IProgressMonitor monitor) throws SftpException {
        return readDirHandle();
    }

    private int sendReadDir() throws SftpCommunicationException {
        FxpReadDir pkt = pktFactory.createReadDir();
        pkt.setHandle(handle);
        connection.sendPacket(pkt);
        return pkt.getId();
    }

    private void sendReadDirPackets(int sendAtOncePackets) throws SftpCommunicationException {
        for (int i = 0; i < sendAtOncePackets; i++) {
            sentPacketIds[i] = sendReadDir();
        }
    }

    private Map readDirHandle() throws SftpException {

        Map attrs = new HashMap();

        // Start sending two packets at once. The minimum number of packets needed to retrieve a 
        // non-empty directory is two: one with a FxpName response, and one with an FxpStatus response
        // with code EOF.
        int sendAtOncePackets = 2;
        
        forever : while (true) {
            sendReadDirPackets(sendAtOncePackets);

            for (int i = 0; i < sendAtOncePackets; i++) {
                SftpPacket response = connection.readPacket(sentPacketIds[i]);
                switch (response.getType()) {
                    case SftpPacket.FXP_STATUS :
                        FxpStatus status = (FxpStatus) response;
                        if (status.getCode() == FxpStatus.FX_EOF) {
                            connection.setMaxIgnorePacketId(sentPacketIds[sendAtOncePackets - 1]);

                            break forever;
                        }
                        defaultStatusHandler((FxpStatus) response);
                        throw new SftpException(
                            "Unexpected non-error status received: " + response);
                    case SftpPacket.FXP_NAME :
                        FxpName names = (FxpName) response;
                        fixupAttributes(names);
                        attrs.putAll(names.getAttrs());
                        break;
                    default :
                        throw new SftpException("Unexpected packet received: " + response);
                }
            }
            // Send more packets at once when no EOF did occur in the previous batch. This way
            // very large directories will be retrieved quicker.
            sendAtOncePackets = Math.min(sendAtOncePackets + 1, MAX_SEND_AT_ONCE_PACKETS);
        } // forever
        return attrs;
    }

    private void fixupAttributes(FxpName names) {
        List fileNames = names.getFiles();
        for (Iterator i = fileNames.iterator(); i.hasNext();) {
            String fileName = (String) i.next();
            fixupAttributes(names.getLongName(fileName), names.getAttrs(fileName));
        }
    }

    private void fixupAttributes(String longName, Attrs a) {
        if (longName != null && a.getType() == 0) {
            char c = longName.charAt(0);
            switch (c) {
                case 'd' :
                    a.setType(Attrs.FILEXFER_TYPE_DIRECTORY);
                    break;
                case '-' :
                    a.setType(Attrs.FILEXFER_TYPE_REGULAR);
                    break;
                case 'l' :
                    a.setType(Attrs.FILEXFER_TYPE_SYMLINK);
                    break;
                case 'c' :
                case 'b' :
                    a.setType(Attrs.FILEXFER_TYPE_SPECIAL);
                    break;
                default :
                    a.setType(Attrs.FILEXFER_TYPE_UNKNOWN);
            }
        }
    }
}
