package org.klomp.sftp.protocol;

import java.io.IOException;
import java.io.InputStream;

import org.eclipse.core.runtime.IProgressMonitor;

import org.klomp.eclipse.team.sftp.SftpException;
import org.klomp.eclipse.team.sftp.SftpPlugin;
import org.klomp.eclipse.team.sftp.client.SftpCommunicationException;
import org.klomp.sftp.packet.Attrs;
import org.klomp.sftp.packet.FxpHandle;
import org.klomp.sftp.packet.FxpOpen;
import org.klomp.sftp.packet.FxpStatus;
import org.klomp.sftp.packet.FxpWrite;
import org.klomp.sftp.packet.SftpPacket;
import org.klomp.sftp.packet.SftpPacketFactory;

public class PlainFile extends SftpFile
{
    private final static int MAX_WRITE_DATA_SIZE = 32000;
    /**
     * Open-mode for file.
     */
    private int mode;

    private PlainFile(SftpConnection connection, String path, byte[] handle, int mode)
    {
        super(connection, path, handle);
        this.mode = mode;
    }

    public static PlainFile openFile(SftpConnection connection, String path, int mode) throws SftpException
    {
        SftpPacketFactory pktFactory = connection.getPktFactory();

        FxpOpen pkt = pktFactory.createOpen();
        pkt.setPath(path);
        pkt.setPflags(mode);
        pkt.setAttrs(new Attrs());
        connection.sendPacket(pkt);
        SftpPacket response = connection.readPacket(pkt.getId());

        switch (response.getType())
        {
            case SftpPacket.FXP_STATUS :
                defaultStatusHandler((FxpStatus) response);
                throw new SftpException("Unexpected non-error status received: " + response);
            case SftpPacket.FXP_HANDLE :
                FxpHandle handlePkt = (FxpHandle) response;
                byte[] handle = handlePkt.getHandle();
                return new PlainFile(connection, path, handle, mode);
            default :
                throw new SftpException("Unexpected packet received: " + response);
        }
    }

    public InputStream getInputStream(long position)
    {
        return new SftpFileInputStream(this, position);
    }

    public void writeFile(InputStream localFile, IProgressMonitor monitor) throws SftpException
    {

        monitor = SftpPlugin.monitorFor(monitor);
        monitor.beginTask(null, 100);
        monitor.subTask(SftpPlugin.formatResourceString("%SftpClient.writeFile", path)); //$NON-NLS-1$

        try
        {
            try
            {
                byte[] data = new byte[MAX_WRITE_DATA_SIZE];
                int rb;
                long fileOffset = 0;
                while ((rb = localFile.read(data)) > 0)
                {
                    writeFileHandle(fileOffset, data, rb);
                    fileOffset += rb;
                }
            }
            catch (IOException e)
            {
                throw new SftpCommunicationException("Reading from local file", e);
            }
        }
        finally
        {
            monitor.done();
        }
    }

    private void writeFileHandle(long fileOffset, byte[] data, int length) throws SftpException
    {

        FxpWrite pkt = pktFactory.createWrite();
        pkt.setHandle(handle);
        pkt.setData(data, 0, length);
        pkt.setOffset(fileOffset);

        connection.sendPacket(pkt);
        FxpStatus response = readStatus(pkt.getId());
        defaultStatusHandler(response);
    }

}
