package org.klomp.sftp.protocol;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.core.runtime.IProgressMonitor;

import org.klomp.eclipse.team.sftp.SftpException;
import org.klomp.eclipse.team.sftp.SftpPlugin;
import org.klomp.eclipse.team.sftp.client.ISftpClientListener;
import org.klomp.eclipse.team.sftp.client.SftpCommunicationException;
import org.klomp.sftp.packet.SftpPacket;
import org.klomp.sftp.packet.SftpPacketFactory;

public abstract class SftpConnection
{
    /**
     * Incoming from remote host.
     */
    protected InputStream input;

    /**
     * Outgoing to remote host.
     */
    protected OutputStream output;

    /**
     * Packet factory.
     */
    protected SftpPacketFactory pktFactory = SftpPacketFactory.create(SftpPacketFactory.VERSION_4);

    /**
     * Packet listener hook, all sent and received packets are notified to this listener, unless the
     * field contains null.
     */
    protected ISftpClientListener listener;

    /**
     * All received packets with ids equal or lower than this id are ignored in the readPacket method.
     * This can be used to ignore spurious packets that were used to speed up transfers.
     */
    private int maxIgnorePacketId;

    private Map receivedPacketQueue = new TreeMap();

    public abstract void open(int timeout, IProgressMonitor monitor) throws SftpException;

    /**
     * Closes this connection.
     * Implemented by subclasses.
     */
    public abstract void close() throws IOException;

    /**
     * Reads an sftp packet.
     */
    public synchronized SftpPacket readPacket(int id) throws SftpCommunicationException
    {
        SftpPacket pkt = readQueuedPacket(id);
        if (pkt != null)
        {
            return pkt;
        }
        else
        {
            try
            {
                return readStreamPacket(id);
            }
            catch (IOException e)
            {
                e.printStackTrace();
                throw new SftpCommunicationException(SftpPlugin.getString("%SftpClient.ErrorReadingPacket"), e); //$NON-NLS-1$, e);
            }
        }
    }

    /**
     * Tries to read a packet with the given id from the queue of packets that were received before.
     * If no packet with the given id is currently in the queue, this method returns null.
     */
    private SftpPacket readQueuedPacket(int id)
    {
        Integer pktId = new Integer(id);
        if (receivedPacketQueue.containsKey(pktId))
        {
            return (SftpPacket) receivedPacketQueue.remove(pktId);
        }
        else
        {
            return null;
        }
    }

    /**
     * Reads a packet from the sftp stream. Only returns when a packet with the given id is received.
     * Stores packets with different id's in the receivedPacketQueue. However packets with ids lowerthan
     * maxIgnorePacketId are ignored.
     */
    private SftpPacket readStreamPacket(int id) throws IOException
    {
        while (true)
        {
            SftpPacket pkt;
            do
            {
                pkt = pktFactory.readPacket(input);
                if (listener != null)
                {
                    listener.packetReceived(pkt);
                }
            }
            while (maxIgnorePacketId != 0 && pkt.getId() <= maxIgnorePacketId);

            if (pkt.getId() == id)
            {
                return pkt;
            }
            else
            {
                receivedPacketQueue.put(new Integer(pkt.getId()), pkt);
            }
        }
    }

    /**
     * Sends an sftp packet.
     */
    public void sendPacket(SftpPacket packet) throws SftpCommunicationException
    {
        try
        {
            packet.writePacket(output);
            output.flush();
        }
        catch (IOException e)
        {
            throw new SftpCommunicationException(SftpPlugin.getString("%SftpClient.ErrorWritingPacket"), e); //$NON-NLS-1$
        }
        if (listener != null)
        {
            listener.packetSent(packet);
        }
    }

    public SftpPacketFactory getPktFactory()
    {
        return pktFactory;
    }

    public int getMaxIgnorePacketId()
    {
        return maxIgnorePacketId;
    }

    public void setMaxIgnorePacketId(int maxIgnorePacketId)
    {
        this.maxIgnorePacketId = maxIgnorePacketId;
    }

    public ISftpClientListener getListener()
    {
        return listener;
    }

    public void setListener(ISftpClientListener listener)
    {
        this.listener = listener;
    }

}
